// Receive Sony TV IR remote control commands - use interrupts
// For PIC18F45K22 on EasyPIC7 development board
// uses 8MHz crystal + PLLx4 to get 32MHz
// connect 3-pin IR receiver output to RB0 - maybe pull-up RB0 for better reliability
// alternatively fit IRclick board into mikroBUS socket 1 and wire RA2 to RB0
// Remove LCD and GLCD from board
// enable PORTC and PORTD LEDs
// Light LEDs in response to Sony TV remote number buttons
// Alternatively connect driver circuit and relays in place of LEDs
// last updated 29 Nov 2015

// timer 1 runs at 1MHz (32MHz osc/4 with 1:8 divider) thus measures milliseconds

// seems to work - but may have bugs, as only quick test performed

// Sony protocol:
// 40kHz 7 bits command (LSB first) then 5 bits address
// this is 12 bit, there are also 15 bit and 20 bit versions of Sony protocol
// start bit = 2.4mS mark, .6mS space
// 1 = 1.2mS mark .6mS space, 0 = .6mS mark .6ms space
// repeat every 45mS if key is held down
// Sony device will not respond to a single message,
//    - so remotes usually send 3 or more repeated messages


// global variables
unsigned int counter;              // store timer 1 value
unsigned int input_data;           // assemble and store incoming bits
unsigned char bit_count;           // count incoming bits
unsigned char current_state;       // 1=idle 2=start 3= capture
unsigned char ir_found;            // flag set in Interrupt to show message ready
unsigned char command_id;          // store for 7 bit command
unsigned char device_id;           // store for 5 bit address (where 1 is TV)

void interrupt(){
    if (INT0IF_bit){                               // RB0 external Interrupt
        switch(current_state){
        case 1:   //Idle,starting
            current_state = 2;
            INTEDG0_bit = 1;                       // interrupt on rising edge
            break;
        case 2:   // start_bit, found rising edge, check length for 2.4ms
            counter = TMR1L;                       // also loads TMR1 H
            counter += (TMR1H * 256);              // combine timer 1 high and low
            if((counter > 2000) && (counter < 2800)){   // acceptable uS values for start
                bit_count = 0;                     // start counting bits in
                input_data = 0;                    // clear old value
                current_state = 3;                 // start receiving bits
            }
            else{                                  // fault signal, reset to idle
                current_state = 1;
                INTEDG0_bit = 0;                   // interrupt on falling edge
            }
            break;
        case 3:   //capture bit, start receiving data
            //check pulse length for "0" (0.6 mS space + 0.6 mS mark)
            // or "1" (0.6 mS space + 1.2 mS mark)
            counter = TMR1L;                       // also loads TMR1 H
            counter += (TMR1H * 256);              // combine timer 1 high and low
            bit_count++;
            input_data >>= 1;                      // make room for 0 bit

            if(counter > 1500){                    // acceptable uS values for zero/one discrimination
                input_data |= 0x8000;              // change 0 to 1
            }
            if (bit_count == 12){                  // complete 12 bits
                ir_found = 1;                      // indicate that a message is ready
                input_data >>= 4;                  // shift 12 bits into lower bits of word
                current_state = 1;                 // ready to start receiving again
            }
            break;
        default: current_state = 1;
        }
        TMR1H = 0x00;                              // reset timer
        TMR1L = 0x00;
        INT0IF_bit = 0;                            // clear Interrupt flag
    }
    // hamdle Timer 1 interrupt
    if (TMR1IF_bit)                                // Timer1 overflow
    {
        current_state = 1;                         // timer has overflowed - so return to idle
        TMR1IF_bit = 0;                            // clear Interrupt flag
    }
}


void main() {
    OSCCON = 0x70;                       // internal 8MHz x4 PLL gives 32 MHz clock
    ANSELA = 0x00;                       // configure all pins as digital
    TRISA = 0x04;                        // IR input may be on RA2 as well as RB0
    LATA = 0;                            // PORTA outputs all low
    ANSELB = 0x00;                       // configure all pins as digital
    TRISB = 0x01;                        // IR input on B0 (and maybe A3 as well)
    LATB = 0;                            // PORT outputs all low
    ANSELC = 0x00;                       // configure all pins as digital
    TRISC = 0x00;                        // address will be shown on PORTC
    LATC = 0;                            // PORT outputs all low
    ANSELD = 0x00;                       // configure all pins as digital
    TRISD = 0x00;                        // command will be shown on PORTD
    LATD = 0;                            // PORT outputs all low

    INTEDG0_bit = 0;                      // interrupt on falling edge

    T1CON  = 0x31;                       // Timer 1 on, FOSC/4, 2:1, internal osc gives mS count
    TMR1H = 0x00;                        // reset timer
    TMR1L = 0x00;
    INTCON = 0xC0;                       // Eneble peripheral and global interrupts
    TMR1IE_bit = 1;
    INT0IE_bit = 1;                      // Enable RB0 Interrupt

    current_state = 1;                   // Idle
    counter = 0;
    ir_found = 0;
 
    while(1){                                       // Infinite loop
        if(ir_found){
            command_id = input_data & 0x7F;
            LATD = command_id;                      // command on PORTD LEDs
            device_id = input_data >> 7;            // break input data to two parts
            LATC = device_id;                       // Address shown on PORTC LEDs
            ir_found = 0;                           // clear the flag for new message
            if(device_id == 1){                     // Sony TV
                switch(command_id){                 // convert number to ASCII hex
                    case 0:                         // "1" button
                        LATA.B0 = !LATA.B0;         // Toggle LED/relay on RA0
                        break;
                    case 1:                         // "2" button
                        LATA.B1 = !LATA.B1;         // Toggle LED/relay on RA1
                        break;
                    case 2:                         // "3" button
                        LATA.B4 = !LATA.B4;         // Toggle LED/relay on RA4
                        break;
                    case 3:                         // "4" button
                        LATA.B5 = !LATA.B5;         // Toggle LED/relay on RA5
                        break;
                    case 8:                         // "9" button
                        LATA = 0xff;                // all LEDs/relays on
                        break;
                    case 9:                         // "0" button
                        LATA = 0;                   // all LEDs/relays off
                        break;
                }
                Delay_ms(100);                      // wait until repeat messages finished...
                                                    // .... to prevent multiple toggles from one button push
                current_state = 1;                  // Idle - kill repeat message in progress
                counter = 0;
                ir_found = 0;
            }
        }
    }
}